import requests
import pandas as pd
import psycopg2

'''
@staticmethod -> A decorator used to declare methods in classes, that do not need a `self` instance or `cls`. There are regular functions defined inside functions for logical grouping
'''
class Extract:
    # Extracts data from a CSV file
    @staticmethod
    def read_csv(filepath: str):
        data = pd.read_csv(filepath)
        return data
    
    # Extracts data from a JSON file
    @staticmethod
    def read_json(filepath: str):
        data = pd.read_json(filepath)
        return data
    
    # Extracts data from an API
    @staticmethod
    def read_api(url: str, headers: dict = None):
        default_kwargs = {
            'headers': headers
        }
        response = requests.get(url, **default_kwargs)
        response.raise_for_status()
        return response.json()
    
    # Extracts data from a PostgreSQL database
    '''
    Read data from a SQL database via URL
    i. url: A database URI to connect to, dtype: str
    ii. table_name: The name of the table to be read, dtype:str
    iii. columns: The columns to be read from the database, dtype:list
    iv. parse_date: The columns to be parsed into datetime columns, dtype: list/dict.
    '''
    @staticmethod
    def read_db_via_url(url: str, table_name: str, columns: list = None, parse_dates: list | dict = None):
        try:
            default_kwargs = {
                'columns': columns,
                'parse_dates': parse_dates
            }

            data = pd.read_sql_table(table_name, con=url, **default_kwargs)
            return data
        except Exception as e:
            print(f"Error: {str(e)}")

        
    '''
    Access database via database parameters. For PostgreSQL databases!
    i. query: A SQL query to show data from database e.g. 'SELECT * FROM employees'
    '''
    @staticmethod
    def read_db_via_params(database:str, user:str, password:str, query, host:str='localhost', port:str='5432'):
        try:
            connection = psycopg2.connect(
                database=database,
                user=user,
                password=password,
                host=host,
                port=port
            )
            data = pd.read_sql(query, connection)
            connection.close() # close connection as data is already retrieved
            return data
        except Exception as e:
            print(f"Error: {str(e)}")




import os
import pandas as pd
from sqlalchemy import create_engine
'''
Load class:
i. Load into a database
ii. Load into a CSV file
iii. Load into a JSON file
iv. Load into an Excel file ( user needs openpyxl module for this )
v. Load into AWS S3 bucket in version 0.2.0
'''
class Load:
    # loads data into a CSV file
    # overwrite = False is a parameter that deny users permissions to overwrite existing files

    # helper function to check if file exists
    @staticmethod
    def _check_file(filepath, overwrite: bool):
        if os.path.exists(filepath) and not overwrite: # checks if filepath already exists in the current directory
            raise FileExistsError(f'The file on {filepath} exists, pass overwrite=True to overwrite file')

    @staticmethod
    def load_csv(data, filepath:str, overwrite: bool = False):
        if not isinstance(data, (pd.DataFrame, pd.Series)): # checks if data is a Pandas DataFrame obj, if not print message
            raise TypeError('Expected data to be a Pandas.DataFrame object')
        
        try:
            Load._check_file(filepath, overwrite)
            data.to_csv(filepath, index=False) # converts data into a CSV file, if it is a Pandas dataframe
            print('CSV file loaded successfully')
        except Exception as e:
            print(f'Error occurred during loading: {str(e)}')

    @staticmethod
    def load_json(data, filepath:str, overwrite: bool=False): # loads data into JSON file
        if not isinstance(data, (pd.DataFrame, pd.Series)):
            raise TypeError('Expected data to be a Pandas.DataFrame object')
        
        try:
            Load._check_file(filepath, overwrite)
            data.to_json(filepath)
            print('JSON file loaded successfully')
        except Exception as e:
            print(f'Error occurred during loading: {str(e)}')

    @staticmethod
    def load_to_excel(data, filepath:str, overwrite: bool = False): # Loads data into an Excel file
        if not isinstance(data, (pd.DataFrame, pd.Series)):
            raise TypeError('Expected data to be a Pandas.DataFrame object')
        
        try:
            Load._check_file(filepath, overwrite)
            data.to_excel(filepath)
            print('Excel file loaded successfully')
        except Exception as e:
            print(f'Error occurred during loading: {str(e)}')

    '''
    load_to_db parameters include:
    i. data - Dataframe to be loaded
    ii. name (str) - name of the db - required
    iii. url (str) - Database connection string - required
    '''
    @staticmethod
    def load_to_db(data, name: str, url: str): # loads data into a database
        if not isinstance(data, (pd.DataFrame, pd.Series)):
            raise TypeError('Expected data needs to be a Pandas.DataFrame object')
        
        try:
            engine = create_engine(url=url) # Creates a connection to the database
            data.to_sql(name=name, con=engine, if_exists='replace') # if table exists, overwrite the table.
            print('Table created successfully!')
        except Exception as e:
            print(f'Error occurred during loading to database: {str(e)}')